@@ -1,6 +1,10 @@
 #' @title Deep Gaussian process emulator construction
 #'
-#' @description This function builds and trains a DGP emulator.
+#' @description
+#'
+#' `r new_badge("updated")`
+#'
+#' This function builds and trains a DGP emulator.
 #'
 #' @param X a matrix where each row is an input training data point and each column represents an input dimension.
 #' @param Y a matrix containing observed training output data. The matrix has its rows being output data points and columns representing
@@ -35,11 +39,18 @@
 #'
 #' Defaults to `"ga"`.
 #' @param share a bool indicating if all input dimensions of a GP node share a common lengthscale. Defaults to `TRUE`.
-#' @param nugget_est a bool or a bool vector that indicates if the nuggets of GP nodes (if any) in the final layer are to be estimated. If a single bool is
-#'     provided, it will be applied to all GP nodes (if any) in the final layer. If a bool vector (which must have a length of `ncol(Y)`) is provided, each
-#'     bool element in the vector will be applied to the corresponding GP node (if any) in the final layer. The value of a bool has following effects:
-#' * `FALSE`: the nugget of the corresponding GP in the final layer is fixed to the corresponding value defined in `nugget` (see below).
-#' * `TRUE`: the nugget of the corresponding GP in the final layer will be estimated with the initial value given by the correspondence in `nugget` (see below).
+#' @param nugget_est `r new_badge("updated")` a bool or a bool vector indicating whether the nuggets of GP nodes in the final layer (or the
+#'     layer feeding the likelihood node) should be estimated. If a bool is provided, it is applied to all GP nodes in that layer.
+#'     If a bool vector is provided, its length must match the number of GP nodes:
+#' - `ncol(Y)` if `likelihood = NULL`
+#' - `2` if `likelihood` is `"Hetero"` or `"NegBin"`
+#' - `1` if `likelihood` is `"Poisson"` or `"Categorical"` with two classes
+#' - the number of classes if `likelihood` is `"Categorical"` with more than two classes.
+#'
+#' Each element of the vector is applied to the corresponding GP node in the final layer (or the layer feeding the
+#'    likelihood node). The value of a bool has following effects:
+#' * `FALSE`: the nugget of the corresponding GP is fixed to the corresponding value defined in `nugget` (see below).
+#' * `TRUE`: the nugget of the corresponding GP will be estimated with the initial value given by the correspondence in `nugget` (see below).
 #'
 #' Defaults to `FALSE`.
 #' @param nugget the initial nugget value(s) of GP nodes (if any) in each layer:
@@ -49,30 +60,44 @@
 #'
 #' Set `nugget` to a small value and the bools in `nugget_est` to `FALSE` for deterministic emulation, where the emulator
 #'    interpolates the training data points. Set `nugget` to a larger value and the bools in `nugget_est` to `TRUE` for stochastic emulation where
-#'    the computer model outputs are assumed to follow a homogeneous Gaussian distribution. Defaults to `1e-6` if `nugget_est = FALSE` and
-#'    `0.01` if `nugget_est = TRUE`. If `likelihood` is not `NULL` and `nugget_est = FALSE`, the nuggets of GPs that feed into the likelihood layer default to
-#'    `1e-4`.
-#' @param scale_est a bool or a bool vector that indicates if the variance of GP nodes (if any) in the final layer are to be estimated. If a single bool is
-#'     provided, it will be applied to all GP nodes (if any) in the final layer. If a bool vector (which must have a length of `ncol(Y)`) is provided, each
-#'     bool element in the vector will be applied to the corresponding GP node (if any) in the final layer. The value of a bool has following effects:
-#' * `FALSE`: the variance of the corresponding GP in the final layer is fixed to the corresponding value defined in `scale` (see below).
-#' * `TRUE`: the variance of the corresponding GP in the final layer will be estimated with the initial value given by the correspondence in `scale` (see below).
+#'    the computer model outputs are assumed to follow a homogeneous Gaussian distribution. Defaults to `1e-6` if `likelihood` is `NULL`.
+#'    If `likelihood` is not `NULL`, the nuggets of GPs that feed into the likelihood layer default to `1e-4`, while those of all other GPs default to `1e-6`.
+#' @param scale_est a bool or a bool vector indicating whether the variances of GP nodes in the final layer (or the
+#'     layer feeding the likelihood node) should be estimated. If a bool is provided, it is applied to all GP nodes in that layer.
+#'     If a bool vector is provided, its length must match the number of GP nodes:
+#' - `ncol(Y)` if `likelihood = NULL`
+#' - `2` if `likelihood` is `"Hetero"` or `"NegBin"`
+#' - `1` if `likelihood` is `"Poisson"` or `"Categorical"` with two classes
+#' - the number of classes if `likelihood` is `"Categorical"` with more than two classes.
+#'
+#' The value of a bool has following effects:
+#' * `FALSE`: the variance of the corresponding GP is fixed to the corresponding value defined in `scale` (see below).
+#' * `TRUE`: the variance of the corresponding GP will be estimated with the initial value given by the correspondence in `scale` (see below).
 #'
 #' Defaults to `TRUE`.
-#' @param scale the initial variance value(s) of GP nodes (if any) in the final layer. If it is a single numeric value, it will be applied to all GP nodes (if any)
-#'    in the final layer. If it is a vector (which must have a length of `ncol(Y)`), each numeric in the vector will be applied to the corresponding GP node
-#'    (if any) in the final layer. Defaults to `1`.
-#' @param connect a bool indicating whether to implement global input connection to the DGP structure. Setting it to `FALSE` may produce a better emulator in some cases at
-#'    the cost of slower training. Defaults to `TRUE`.
+#' @param scale the initial variance value(s) of GP nodes in the final layer (or the
+#'    layer feeding the likelihood node). If it is a single numeric value, it will be applied to all GP nodes
+#'    in the final layer (or the layer feeding the likelihood node). If it is a vector, its length must match the number of GP nodes:
+#' - `ncol(Y)` if `likelihood = NULL`
+#' - `2` if `likelihood` is `"Hetero"` or `"NegBin"`
+#' - `1` if `likelihood` is `"Poisson"` or `"Categorical"` with two classes
+#' - the number of classes if `likelihood` is `"Categorical"` with more than two classes.
+#'
+#' Each numeric in the vector will be applied to the corresponding GP node.
+#'
+#' Defaults to `1`.
+#' @param connect a bool indicating whether to apply global input connections in the DGP structure. Setting this to `FALSE` may yield
+#'     a better emulator in some cases. When set to `NULL`, the value defaults to `FALSE` if `likelihood = "Categorical"` and to `TRUE` otherwise.
+#'     Defaults to `NULL`.
 #' @param likelihood the likelihood type of a DGP emulator:
 #' 1. `NULL`: no likelihood layer is included in the emulator.
 #' 2. `"Hetero"`: a heteroskedastic Gaussian likelihood layer is added for stochastic emulation where the computer model outputs are assumed to follow a heteroskedastic Gaussian distribution
 #'    (i.e., the computer model outputs have input-dependent noise).
 #' 3. `"Poisson"`: a Poisson likelihood layer is added for emulation where the computer model outputs are counts and a Poisson distribution is used to model them.
 #' 4. `"NegBin"`: a negative Binomial likelihood layer is added for emulation where the computer model outputs are counts and a negative Binomial distribution is used to capture dispersion variability in input space.
-#' 5. `r new_badge("new")` `"Categorical"`: a categorical likelihood layer is added for emulation (classification), where the computer model output is categorical.
+#' 5. `"Categorical"`: a categorical likelihood layer is added for emulation (classification), where the computer model output is categorical.
 #'
-#' When `likelihood` is not `NULL`, the value of `nugget_est` is overridden by `FALSE`. Defaults to `NULL`.
+#' Defaults to `NULL`.
 #' @param training a bool indicating if the initialized DGP emulator will be trained.
 #'     When set to `FALSE`, [dgp()] returns an untrained DGP emulator, to which one can apply [summary()] to inspect its specifications
 #'     or apply [predict()] to check its emulation performance before training. Defaults to `TRUE`.
@@ -80,9 +105,9 @@
 #'     Defaults to `TRUE`.
 #' @param check_rep a bool indicating whether to check for repetitions in the dataset, i.e., if one input
 #'     position has multiple outputs. Defaults to `TRUE`.
-#' @param vecchia `r new_badge("new")` a bool indicating whether to use Vecchia approximation for large-scale DGP emulator construction and prediction. Defaults to `FALSE`.
-#' @param M `r new_badge("new")` the size of the conditioning set for the Vecchia approximation in the DGP emulator training. Defaults to `25`.
-#' @param ord `r new_badge("new")` an R function that returns the ordering of the input to each GP node contained in the DGP emulator for the Vecchia approximation. The
+#' @param vecchia a bool indicating whether to use Vecchia approximation for large-scale DGP emulator construction and prediction. Defaults to `FALSE`.
+#' @param M the size of the conditioning set for the Vecchia approximation in the DGP emulator training. Defaults to `25`.
+#' @param ord an R function that returns the ordering of the input to each GP node contained in the DGP emulator for the Vecchia approximation. The
 #'    function must satisfy the following basic rules:
 #' * the first argument represents the input to a GP node scaled by its lengthscales.
 #' * the output of the function is a vector of indices that gives the ordering of the input to the GP node.
@@ -102,36 +127,13 @@
 #'     are used. Defaults to `NULL`. This argument is only used when `training = TRUE`.
 #' @param B the number of imputations used to produce predictions. Increase the value to refine the representation of imputation uncertainty.
 #'     Defaults to `10`.
-#' @param internal_input_idx `r lifecycle::badge("deprecated")` The argument will be removed in the next release. To set up connections of emulators for linked emulations,
-#'     please use the updated [lgp()] function instead.
-#'
-#' Column indices of `X` that are generated by the linked emulators in the preceding layers.
-#'     Set `internal_input_idx = NULL` if the DGP emulator is in the first layer of a system or all columns in `X` are
-#'     generated by the linked emulators in the preceding layers. Defaults to `NULL`.
-#' @param linked_idx `r lifecycle::badge("deprecated")` The argument will be removed in the next release. To set up connections of emulators for linked emulation,
-#'     please use the updated [lgp()] function instead.
-#'
-#' Either a vector or a list of vectors:
-#' * If `linked_idx` is a vector, it gives indices of columns in the pooled output matrix (formed by column-combined outputs of all
-#'   emulators in the feeding layer) that feed into the DGP emulator. The length of the vector shall equal to the length of `internal_input_idx`
-#'   when `internal_input_idx` is not `NULL`. If the DGP emulator is in the first layer of a linked emulator system, the vector gives the column indices of the global
-#'   input (formed by column-combining all input matrices of emulators in the first layer) that the DGP emulator will use. If the DGP emulator is to be used in both the first
-#'   and subsequent layers, one should initially set `linked_idx` to the appropriate values for the situation where the emulator is not in the first layer. Then, use the
-#'   function [set_linked_idx()] to reset the linking information when the emulator is in the first layer.
-#' * When the DGP emulator is not in the first layer of a linked emulator system, `linked_idx` can be a list that gives the information on connections
-#'   between the DGP emulator and emulators in all preceding layers. The length of the list should equal to the number of layers before
-#'   the DGP emulator. Each element of the list is a vector that gives indices of columns in the pooled output matrix (formed by column-combined outputs
-#'   of all emulators) in the corresponding layer that feed into the DGP emulator. If the DGP emulator has no connections to any emulator in a certain layer,
-#'   set `NULL` in the corresponding position of the list. The order of input dimensions in `X[,internal_input_idx]` should be consistent with `linked_idx`.
-#'   For example, a DGP emulator in the 4th-layer that is fed by the output dimension 2 and 4 of emulators in layer 2 and all output dimension 1 to 3 of
-#'   emulators in layer 3 should have `linked_idx = list( NULL, c(2,4), c(1,2,3) )`. In addition, the first and second columns of `X[,internal_input_idx]`
-#'   should correspond to the output dimensions 2 and 4 from layer 2, and the third to fifth columns of `X[,internal_input_idx]` should
-#'   correspond to the output dimensions 1 to 3 from layer 3.
-#'
-#' Set `linked_idx = NULL` if the DGP emulator will not be used for linked emulations. However, if this is no longer the case, one can use [set_linked_idx()]
-#' to add linking information to the DGP emulator. Defaults to `NULL`.
 #' @param id an ID to be assigned to the DGP emulator. If an ID is not provided (i.e., `id = NULL`), a UUID (Universally Unique Identifier) will be automatically generated
 #'    and assigned to the emulator. Default to `NULL`.
+#' @param decouple `r new_badge("new")` A boolean indicating whether the model parameters for the heteroskedastic Gaussian likelihood, negative Binomial likelihood, and
+#'    categorical likelihood (when the number of categories is greater than 2) should be modeled using separate deep Gaussian process hierarchies when `depth` is greater than 2.
+#'    Defaults to `FALSE`.
+#' @param link `r new_badge("new")` the link function used for binary classification when `likelihood = "Categorical"`. Supported options are `"logit"` and `"probit"`.
+#'    Defaults to `"logit"`.
 #'
 #' @return An S3 class named `dgp` that contains five slots:
 #' * `id`: A number or character string assigned through the `id` argument.
@@ -145,17 +147,11 @@
 #'      - `lengthscales`: a vector of lengthscales in the kernel function.
 #'      - `scale`: the variance value in the kernel function.
 #'      - `nugget`: the nugget value in the kernel function.
-#'   2. `r lifecycle::badge("deprecated")` `internal_dims`: the column indices of `X` that correspond to the linked emulators in the preceding layers of a linked system.
-#'      **The slot will be removed in the next release**.
-#'   3. `r lifecycle::badge("deprecated")` `external_dims`: the column indices of `X` that correspond to global inputs to the linked system of emulators. It is shown
-#'      as `FALSE` if `internal_input_idx = NULL`. **The slot will be removed in the next release**.
-#'   4. `r lifecycle::badge("deprecated")` `linked_idx`: the value passed to argument `linked_idx`. It is shown as `FALSE` if the argument `linked_idx` is `NULL`.
-#'      **The slot will be removed in the next release**.
-#'   5. `seed`: the random seed generated to produce imputations. This information is stored for reproducibility when the DGP emulator (that was saved by [write()]
+#'   2. `seed`: the random seed generated to produce imputations. This information is stored for reproducibility when the DGP emulator (that was saved by [write()]
 #'      with the light option `light = TRUE`) is loaded back to R by [read()].
-#'   6. `B`: the number of imputations used to generate the emulator.
-#'   7. `r new_badge("new")` `vecchia`: whether the Vecchia approximation is used for the GP emulator training.
-#'   8. `r new_badge("new")` `M`: the size of the conditioning set for the Vecchia approximation in the DGP emulator training. `M` is generated only when `vecchia = TRUE`.
+#'   3. `B`: the number of imputations used to generate the emulator.
+#'   4. `vecchia`: whether the Vecchia approximation is used for the GP emulator training.
+#'   5. `M`: the size of the conditioning set for the Vecchia approximation in the DGP emulator training. `M` is generated only when `vecchia = TRUE`.
 #' * `constructor_obj`: a 'python' object that stores the information of the constructed DGP emulator.
 #' * `container_obj`: a 'python' object that stores the information for the linked emulation.
 #' * `emulator_obj`: a 'python' object that stores the information for the predictions from the DGP emulator.
@@ -231,44 +227,20 @@
 #' @md
 #' @export
 dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.0, bounds = NULL, prior = 'ga', share = TRUE,
-                nugget_est = FALSE, nugget = NULL, scale_est = TRUE, scale = 1., connect = TRUE,
+                nugget_est = FALSE, nugget = NULL, scale_est = TRUE, scale = 1., connect = NULL,
                 likelihood = NULL, training =TRUE, verb = TRUE, check_rep = TRUE, vecchia = FALSE, M = 25, ord = NULL, N = ifelse(vecchia, 200, 500), cores = 1, blocked_gibbs = TRUE,
-                ess_burn = 10, burnin = NULL, B = 10, internal_input_idx = NULL, linked_idx = NULL, id = NULL) {
+                ess_burn = 10, burnin = NULL, B = 10, id = NULL, decouple = FALSE, link = "logit") {
   if ( is.null(pkg.env$dgpsi) ) {
     init_py(verb = F)
     if (pkg.env$restart) return(invisible(NULL))
   }
 
-  if (!is.null(internal_input_idx)) {
-    lifecycle::deprecate_warn(
-      when = "2.5.0",
-      what = "dgp(internal_input_idx)",
-      details = c(i = "The argument will be dropped in the next release.",
-                  i = "To set up connections of DGPs for linked emulation, please use the updated `lgp()` function instead."
-      )
-    )
-  }
-
-  if (!is.null(linked_idx)) {
-    lifecycle::deprecate_warn(
-      when = "2.5.0",
-      what = "dgp(linked_idx)",
-      details = c(i = "The argument will be dropped in the next release.",
-                  i = "To set up connections of DGPs for linked emulation, please use the updated `lgp()` function instead."
-      )
-    )
-  }
-
   if (  is.null(nugget) ) {
-    if ( all(nugget_est) ) {
-      nugget = 0.01
+    if ( is.null(likelihood) ) {
+      nugget = 1e-6
     } else {
-        if ( is.null(likelihood) ) {
-          nugget = 1e-6
-        } else {
-          nugget = c(rep(1e-6, depth-2), 1e-4)
-        }
-      }
+      nugget = c(rep(1e-6, depth-2), 1e-4)
+    }
   }
 
   if ( !is.matrix(X)&!is.vector(X) ) stop("'X' must be a vector or a matrix.", call. = FALSE)
@@ -286,6 +258,17 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
 
   if ( !is.null(likelihood) ){
     if (likelihood!='Hetero' &  likelihood!='Poisson' & likelihood!='NegBin' & likelihood!='Categorical' ) stop("The provided 'likelihood' is not supported.", call. = FALSE)
+    if (is.null(connect)){
+      if (likelihood=='Categorical'){
+        connect <- FALSE
+      } else {
+        connect <- TRUE
+      }
+    }
+  } else {
+    if (is.null(connect)){
+      connect <- TRUE
+    }
   }
 
   N <- as.integer(N)
@@ -310,8 +293,6 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
     burnin <- as.integer(burnin)
   }
 
-  linked_idx_py <- linked_idx_r_to_py(linked_idx)
-
   #If struc is NULL
     depth <- as.integer(depth)
     if ( depth < 2 ) stop("'depth' must >= 2. Use gp() if you want a single-layered DGP.", call. = FALSE)
@@ -429,7 +410,15 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
 
       if ( likelihood == 'Hetero'|likelihood == 'NegBin' ) {
         #nugget <- rep(1e-6, 2)
-        nugget_est <- rep(FALSE, 2)
+        #nugget_est <- rep(FALSE, 2)
+        if ( length(nugget_est)==1 ) {
+          nugget_est <- rep(nugget_est, 2)
+        } else {
+          if ( length(nugget_est)!=2 ) {
+            stop(sprintf("length(nugget_est) should equal %i.", 2), call. = FALSE)
+          }
+        }
+
         if ( length(scale_est)==1 ) {
           scale_est <- rep(scale_est, 2)
         } else {
@@ -447,17 +436,29 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
         }
       } else if ( likelihood == 'Poisson' ) {
         #nugget <- 1e-6
-        nugget_est <- FALSE
+        if ( length(nugget_est)!=1 ) {
+          stop(sprintf("length(nugget_est) should equal %i.", 1), call. = FALSE)
+        }
         if ( length(scale_est)!=1 ) stop(sprintf("length(scale_est) should equal %i.", 1), call. = FALSE)
         if ( length(scale)!=1 ) stop(sprintf("length(scale) should equal %i.", 1), call. = FALSE)
       } else if ( likelihood == 'Categorical' ) {
         num_class <- length(unique(Y[,1]))
         if (num_class==2) {
-          nugget_est <- FALSE
+          if ( length(nugget_est)!=1 ) {
+            stop(sprintf("length(nugget_est) should equal %i.", 1), call. = FALSE)
+          }
           if ( length(scale_est)!=1 ) stop(sprintf("length(scale_est) should equal %i.", 1), call. = FALSE)
           if ( length(scale)!=1 ) stop(sprintf("length(scale) should equal %i.", 1), call. = FALSE)
+          if ( link!='logit' & link!='probit' ) stop("'link' can only be either 'logit' or 'probit'.", call. = FALSE)
         } else {
-          nugget_est <- rep(FALSE, num_class)
+          if ( length(nugget_est)==1 ) {
+            nugget_est <- rep(nugget_est, num_class)
+          } else {
+            if ( length(nugget_est)!=num_class ) {
+              stop(sprintf("length(nugget_est) should equal %i.", num_class), call. = FALSE)
+            }
+          }
+
           if ( length(scale_est)==1 ) {
             scale_est <- rep(scale_est, num_class)
           } else {
@@ -479,18 +480,6 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
       no_gp_layer = depth - 1
     }
 
-    if( !is.null(internal_input_idx) ) {
-      external_input_idx <- setdiff(1:n_dim_X, internal_input_idx)
-      if ( length(external_input_idx) == 0) {
-        internal_input_idx = NULL
-        external_input_idx = NULL
-      } else {
-        internal_input_idx <- reticulate::np_array(as.integer(internal_input_idx - 1))
-        external_input_idx <- reticulate::np_array(as.integer(external_input_idx - 1))
-      }
-    } else {
-      external_input_idx = NULL
-    }
 
     if ( isTRUE(verb) ) message(sprintf("Auto-generating a %i-layered DGP structure ...", depth ), appendLF = FALSE)
 
@@ -518,8 +507,7 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
             } else {
               length_scale <- rep(lengthscale[l], n_dim_X)
             }
-            layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k],
-                                                 input_dim = internal_input_idx, connect = external_input_idx)
+            layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k])
           } else {
             if ( connect ) {
               if ( isTRUE(share) ){
@@ -528,19 +516,30 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
                 length_scale <- rep(lengthscale[l], n_dim_X+node)
               }
               connect_info <- as.integer(1:n_dim_X - 1)
-              if (!is.null(internal_input_idx)){
-                idx <- reticulate::py_to_r(internal_input_idx)+1
-                connect_info <- c(connect_info[idx],connect_info[-idx])
+
+              if ( (likelihood == 'Hetero' && decouple) ||
+                   (likelihood == 'NegBin' && decouple) ||
+                   (likelihood == 'Categorical' && decouple && length(unique(Y[,1])) > 2)){
+                layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k],
+                                                     input_dim = reticulate::np_array(as.integer(((1+(k-1)*node):(k*node)) - 1)), connect = reticulate::np_array(connect_info) )
+              } else {
+                layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k],
+                                                     connect = reticulate::np_array(connect_info) )
               }
-              layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k],
-                                                   connect = reticulate::np_array(connect_info) )
             } else {
               if ( isTRUE(share) ){
                 length_scale <- lengthscale[l]
               } else {
                 length_scale <- rep(lengthscale[l], node)
               }
-              layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k])
+              if ( (likelihood == 'Hetero' && decouple) ||
+                   (likelihood == 'NegBin' && decouple) ||
+                   (likelihood == 'Categorical' && decouple && length(unique(Y[,1])) > 2)){
+                layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k],
+                                                     input_dim = reticulate::np_array(as.integer(((1+(k-1)*node):(k*node)) - 1)))
+              } else {
+                layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, scale = scale[k], scale_est = scale_est[k], nugget = nugget[l], nugget_est = nugget_est[k])
+              }
             }
           }
         } else {
@@ -550,8 +549,18 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
               } else {
                 length_scale <- rep(lengthscale[l], n_dim_X)
               }
-              layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
-                                                   input_dim = internal_input_idx, connect = external_input_idx)
+              if ( (likelihood == 'Hetero' && decouple) ||
+                   (likelihood == 'NegBin' && decouple) ){
+                  layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l])
+                  layer_l[[k + no_kerenl]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l])
+              } else if ( likelihood == 'Categorical' && decouple && length(unique(Y[,1])) > 2 ){
+                num_cat <- length(unique(Y[,1]))
+                for (i in 1:num_cat){
+                  layer_l[[k+(i-1)*no_kerenl]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l])
+                }
+              } else {
+              layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l])
+              }
             } else {
               if ( connect ) {
                 if ( isTRUE(share) ){
@@ -560,19 +569,44 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
                   length_scale <- rep(lengthscale[l], n_dim_X+node)
                 }
                 connect_info <- as.integer(1:n_dim_X - 1)
-                if (!is.null(internal_input_idx)){
-                  idx <- reticulate::py_to_r(internal_input_idx)+1
-                  connect_info <- c(connect_info[idx],connect_info[-idx])
+
+                if ( (likelihood == 'Hetero' && decouple) ||
+                     (likelihood == 'NegBin' && decouple) ){
+                  layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
+                                                       input_dim = reticulate::np_array(as.integer((1:no_kerenl) - 1)), connect = reticulate::np_array(connect_info))
+                  layer_l[[k + no_kerenl]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
+                                                                   input_dim = reticulate::np_array(as.integer(((1+no_kerenl):(2*no_kerenl)) - 1)), connect = reticulate::np_array(connect_info))
+                } else if ( likelihood == 'Categorical' && decouple && length(unique(Y[,1])) > 2 ){
+                  num_cat <- length(unique(Y[,1]))
+                  for (i in 1:num_cat){
+                    layer_l[[k+(i-1)*no_kerenl]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
+                                                                         input_dim = reticulate::np_array(as.integer(((1+(i-1)*no_kerenl):(i*no_kerenl)) - 1)), connect = reticulate::np_array(connect_info))
+                  }
+                } else {
+                  layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
+                                                       connect = reticulate::np_array(connect_info))
                 }
-                layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
-                                                     connect = reticulate::np_array(connect_info))
               } else {
                 if ( isTRUE(share) ){
                   length_scale <- lengthscale[l]
                 } else {
                   length_scale <- rep(lengthscale[l], node)
                 }
-                layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l])
+                if ( (likelihood == 'Hetero' && decouple) ||
+                     (likelihood == 'NegBin' && decouple) ){
+                  layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
+                                                       input_dim = reticulate::np_array(as.integer((1:no_kerenl) - 1)))
+                  layer_l[[k + no_kerenl]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
+                                                                   input_dim = reticulate::np_array(as.integer(((1+no_kerenl):(2*no_kerenl)) - 1)))
+                } else if ( likelihood == 'Categorical' && decouple && length(unique(Y[,1])) > 2 ){
+                  num_cat <- length(unique(Y[,1]))
+                  for (i in 1:num_cat){
+                    layer_l[[k+(i-1)*no_kerenl]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l],
+                                                                         input_dim = reticulate::np_array(as.integer(((1+(i-1)*no_kerenl):(i*no_kerenl)) - 1)))
+                  }
+                } else {
+                  layer_l[[k]] <- pkg.env$dgpsi$kernel(length = reticulate::np_array(length_scale), bds = bds, name = name[l], prior_name = prior, nugget = nugget[l])
+                }
               }
             }
           }
@@ -587,7 +621,7 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
       } else if ( likelihood == 'NegBin' ) {
         struc[[depth]] <- c(pkg.env$dgpsi$NegBin())
       } else if ( likelihood == 'Categorical' ) {
-        struc[[depth]] <- c(pkg.env$dgpsi$Categorical(num_class))
+        struc[[depth]] <- c(pkg.env$dgpsi$Categorical(num_class, link = link))
       }
     }
     if ( isTRUE(verb) ) {
@@ -626,19 +660,18 @@ dgp <- function(X, Y, depth = 2, node = ncol(X), name = 'sexp', lengthscale = 1.
 
   res <- list()
   res[['id']] <- if (is.null(id)) uuid::UUIDgenerate() else id
-  res[['data']][['X']] <- unname(X)
-  res[['data']][['Y']] <- unname(Y)
+  rownames(X) <- NULL
+  rownames(Y) <- NULL
+  res[['data']][['X']] <- X
+  res[['data']][['Y']] <- Y
   res[['specs']] <- extract_specs(est_obj, "dgp")
-  res[['specs']][['internal_dims']] <- if( is.null(internal_input_idx) ) 1:n_dim_X else as.integer(reticulate::py_to_r(internal_input_idx)+1)
-  res[['specs']][['external_dims']] <- if( is.null(internal_input_idx) ) FALSE else as.integer(reticulate::py_to_r(external_input_idx)+1)
-  res[['specs']][['linked_idx']] <- if ( is.null(linked_idx) ) FALSE else linked_idx
   res[['specs']][['vecchia']] <- vecchia
   res[['specs']][['M']] <- M
   res[['constructor_obj']] <- obj
   seed <- sample.int(100000, 1)
   set_seed(seed)
   res[['emulator_obj']] <- pkg.env$dgpsi$emulator(all_layer = est_obj, N = B, block = blocked_gibbs)
-  res[['container_obj']] <- pkg.env$dgpsi$container(est_obj, linked_idx_py, block = blocked_gibbs)
+  res[['container_obj']] <- pkg.env$dgpsi$container(est_obj, NULL, block = blocked_gibbs)
   res[['specs']][['seed']] <- seed
   res[['specs']][['B']] <- B
 
@@ -695,6 +728,7 @@ continue <- function(object, N = NULL, cores = 1, ess_burn = 10, verb = TRUE, bu
   if ( !inherits(object,"dgp") ){
     stop("'object' must be an instance of the 'dgp' class.", call. = FALSE)
   }
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
 
   if( is.null(N) ) {
     if (object[['specs']][['vecchia']]) {
@@ -744,9 +778,6 @@ continue <- function(object, N = NULL, cores = 1, ess_burn = 10, verb = TRUE, bu
   new_object[['data']][['X']] <- object$data$X
   new_object[['data']][['Y']] <- object$data$Y
   new_object[['specs']] <- extract_specs(est_obj, "dgp")
-  new_object[['specs']][['internal_dims']] <- object[['specs']][['internal_dims']]
-  new_object[['specs']][['external_dims']] <- object[['specs']][['external_dims']]
-  new_object[['specs']][['linked_idx']] <- if ( is.null(linked_idx) ) FALSE else linked_idx_py_to_r(linked_idx)
   new_object[['specs']][['vecchia']] <- object[['specs']][['vecchia']]
   new_object[['specs']][['M']] <- object[['specs']][['M']]
   new_object[['constructor_obj']] <- constructor_obj_cp
