@@ -1,10 +1,7 @@
+.panel_x <- .panel_y <- NULL
 #' @title Validation and diagnostic plots for a sequential design
 #'
-#' @description
-#'
-#' `r new_badge("updated")`
-#'
-#' This function draws diagnostic and validation plots for a sequential design of a (D)GP emulator or a bundle of (D)GP emulators.
+#' @description This function draws diagnostic and validation plots for a sequential design of a (D)GP emulator or a bundle of (D)GP emulators.
 #'
 #' @param object can be one of the following emulator classes:
 #' * the S3 class `gp`.
@@ -17,7 +14,7 @@
 #' Defaults to `"rmse"`.
 #' @param log a bool indicating whether to plot RMSEs, log-losses (for DGP emulators with categorical likelihoods), or custom evaluation metrics on a log scale when `type = "rmse"`.
 #'     Defaults to `FALSE`.
-#' @param emulator `r new_badge("updated")` an index or vector of indices of emulators packed in `object`. This argument is only used if `object` is an instance of the `bundle` class. When set to `NULL`, all
+#' @param emulator an index or vector of indices of emulators packed in `object`. This argument is only used if `object` is an instance of the `bundle` class. When set to `NULL`, all
 #'     emulators in the bundle are drawn. Defaults to `NULL`.
 #' @param ... N/A.
 #'
@@ -48,14 +45,17 @@ draw.gp <- function(object, type = 'rmse', log = FALSE, ...){
   if ( !inherits(object,"gp") ){
     stop("'object' must be an instance of the 'gp' class.", call. = FALSE)
   }
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   if ( !("design" %in% names(object)) ) stop("'object' must contain the 'design' slot created from design() to be used by draw().", call. = FALSE)
 
   if ( type == 'design' ) {
     design_data <- object$data$X
     total_N <- nrow(design_data)
     design_data <- as.data.frame(design_data)
-    for (l in 1:length(design_data)) {
-      names(design_data)[l] <- paste('X', l, sep="")
+    if (is.null(colnames(object$data$X))){
+      for (l in 1:length(design_data)) {
+        names(design_data)[l] <- paste('X', l, sep="")
+      }
     }
     wave <- c()
     seq_N <- 0
@@ -73,11 +73,16 @@ draw.gp <- function(object, type = 'rmse', log = FALSE, ...){
 
     p <- pair_design(design_data)
 
+    caption_text <- if (is.null(colnames(object$data$X))) {
+      'Xi = Input dimension i of the GP emulator'
+    } else {
+      NULL
+    }
+
     p_patch <- patchwork::wrap_plots(p) +
       patchwork::plot_annotation(
       title = 'Sequential Design',
-      caption =
-        'Xi = Input dimension i of the GP emulator'
+      caption = caption_text
     )
   } else if ( type == 'rmse' ){
     total_N <- nrow(object$data$X)
@@ -142,14 +147,16 @@ draw.gp <- function(object, type = 'rmse', log = FALSE, ...){
         patchwork::plot_annotation(
           title = 'Sequential Design Validation'
         ) +
-        patchwork::plot_layout(guides = 'collect') & ggplot2::theme(legend.position='bottom')
+        patchwork::plot_layout(guides = 'collect') +
+        patchwork::plot_annotation(theme = ggplot2::theme(legend.position = "bottom"))
     } else {
       p_patch <- p_patch +
         patchwork::plot_annotation(
           title = 'Sequential Design Validation',
           caption = 'RMSE = Root Mean Squared Error'
         ) +
-        patchwork::plot_layout(guides = 'collect') & ggplot2::theme(legend.position='bottom')
+        patchwork::plot_layout(guides = 'collect') +
+        patchwork::plot_annotation(theme = ggplot2::theme(legend.position = "bottom"))
     }
   } else {
     stop("The provided 'type' is not supported.", call. = FALSE)
@@ -169,14 +176,17 @@ draw.dgp <- function(object, type = 'rmse', log = FALSE, ...){
   if ( !inherits(object,"dgp") ){
     stop("'object' must be an instance of the 'dgp' class.", call. = FALSE)
   }
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   if ( !("design" %in% names(object)) ) stop("'object' must contain the 'design' slot created from design() to be used by draw().", call. = FALSE)
 
   if ( type == 'design' ) {
     design_data <- object$data$X
     total_N <- nrow(design_data)
     design_data <- as.data.frame(design_data)
-    for (l in 1:length(design_data)) {
-      names(design_data)[l] <- paste('X', l, sep="")
+    if (is.null(colnames(object$data$X))){
+      for (l in 1:length(design_data)) {
+        names(design_data)[l] <- paste('X', l, sep="")
+      }
     }
     wave <- c()
     seq_N <- 0
@@ -191,14 +201,18 @@ draw.dgp <- function(object, type = 'rmse', log = FALSE, ...){
     }
     wave <- c(rep("Initial", total_N-seq_N), wave)
     design_data$Design <- wave
-
     p <- pair_design(design_data)
 
+    caption_text <- if (is.null(colnames(object$data$X))) {
+      'Xi = Input dimension i of the DGP emulator'
+    } else {
+      NULL
+    }
+
     p_patch <- patchwork::wrap_plots(p) +
       patchwork::plot_annotation(
         title = 'Sequential Design',
-        caption =
-          'Xi = Input dimension i of the DGP emulator'
+        caption = caption_text
       )
   } else if ( type == 'rmse' ){
     if (object$constructor_obj$all_layer[[object$constructor_obj$n_layer]][[1]]$name == "Categorical") {
@@ -274,7 +288,8 @@ draw.dgp <- function(object, type = 'rmse', log = FALSE, ...){
       }
     }
     p_patch <- patchwork::wrap_plots(p_list) +
-      patchwork::plot_layout(guides = 'collect') & ggplot2::theme(legend.position='bottom')
+      patchwork::plot_layout(guides = 'collect') +
+      patchwork::plot_annotation(theme = ggplot2::theme(legend.position = "bottom"))
     if ( cust ){
       if ( output_D==ncol(object$data$Y) ){
         p_patch <- p_patch +
@@ -323,6 +338,7 @@ draw.bundle <- function(object, type = 'rmse', log = FALSE, emulator = NULL, ...
   if ( !inherits(object,"bundle") ){
     stop("'object' must be an instance of the 'bundle' class.", call. = FALSE)
   }
+  if ( reticulate::py_is_null_xptr(object$emulator1$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulators in the bundle or, if the bundle was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   if ( !("design" %in% names(object)) ) stop("'object' must contain the 'design' slot created from design() to be used by draw().", call. = FALSE)
 
   if (is.null(emulator)){
@@ -336,8 +352,10 @@ draw.bundle <- function(object, type = 'rmse', log = FALSE, emulator = NULL, ...
       design_data <- object$data$X[[paste('emulator', emulator[emu], sep="")]]
       total_N <- nrow(design_data)
       design_data <- as.data.frame(design_data)
-      for (l in 1:length(design_data)) {
-        names(design_data)[l] <- paste('X', l, sep="")
+      if (is.null(colnames(object$data$X[[paste('emulator', emulator[emu], sep="")]]))){
+        for (l in 1:length(design_data)) {
+          names(design_data)[l] <- paste('X', l, sep="")
+        }
       }
       wave <- c()
       seq_N <- 0
@@ -354,12 +372,13 @@ draw.bundle <- function(object, type = 'rmse', log = FALSE, emulator = NULL, ...
       design_data$Design <- wave
       p_list[[emu]] <- pair_design(design_data) +
         ggplot2::ggtitle(sprintf("Emulator %i", emulator[emu])) +
-        ggplot2::theme(plot.title = ggplot2::element_text(size=10))
+        ggplot2::theme(plot.title = ggplot2::element_text(size=10)) +
+        ggplot2::theme(legend.position = "bottom")
 
     }
 
     p_patch <- patchwork::wrap_plots(p_list) +
-      patchwork::plot_layout() & ggplot2::theme(legend.position='bottom')
+      patchwork::plot_layout()
 
     p_patch <- p_patch +
       patchwork::plot_annotation(
@@ -457,7 +476,8 @@ draw.bundle <- function(object, type = 'rmse', log = FALSE, emulator = NULL, ...
     }
 
     p_patch <- patchwork::wrap_plots(p_list) +
-      patchwork::plot_layout(guides = 'collect') & ggplot2::theme(legend.position='bottom')
+      patchwork::plot_layout(guides = 'collect') +
+      patchwork::plot_annotation(theme = ggplot2::theme(legend.position = "bottom"))
 
     if ( cust ){
       p_patch <- p_patch +
@@ -489,8 +509,8 @@ pair_design <- function(dat){
            "#CAB2D6", "#FDBF6F", "khaki2", "maroon", "orchid1", "deeppink1", "blue1", "steelblue4", "darkturquoise", "green1", "yellow4", "yellow3",
            "darkorange4", "brown")
   p <- ggplot2::ggplot(dat) +
-    ggplot2::geom_point(ggplot2::aes_(x = ~.panel_x, y = ~.panel_y, shape=~Design, color =~Design)) +
-    ggforce::facet_matrix(ggplot2::vars(names(dat)[1:(length(dat)-1)])) +
+    ggplot2::geom_point(ggplot2::aes(x = .panel_x, y = .panel_y, shape=.data$Design, color =.data$Design)) +
+    ggforce::facet_matrix(ggplot2::vars(names(dat)[1:(length(dat)-1)]), labeller = ggplot2::label_value) +
     ggplot2::scale_shape_manual(values = c(17,rep(16,length(unique(dat$Design))-1)), breaks=unique(dat$Design)) +
     ggplot2::scale_colour_manual(values = ggplot2::alpha(c25,0.7), breaks=unique(dat$Design)) +
     ggplot2::theme(
@@ -512,13 +532,13 @@ draw_seq_design <- function(dat, log, target, cust, is.categorical = FALSE) {
 
   if ( !is.null(target) ) {
     p <- p +
-      ggplot2::geom_hline(data = target, mapping = ggplot2::aes_(yintercept=~val, group=~Target, linetype=~Target), alpha=0.8, color="gray20", size = 0.5) +
+      ggplot2::geom_hline(data = target, mapping = ggplot2::aes(yintercept=.data$val, group=.data$Target, linetype=.data$Target), alpha=0.8, color="gray20", linewidth = 0.5) +
       ggplot2::scale_linetype_manual(values=c("dashed", "dotdash", "twodash", "dotted", "solid", "longdash"))
   }
 
   p <- p +
-    ggplot2::geom_line(dat, mapping=ggplot2::aes_(x=~N, y=~rmse, group=~Design, color=~Design), alpha=0.8, stat = "unique") +
-    ggplot2::geom_point(dat, mapping=ggplot2::aes_(x=~N, y=~rmse, group=~Design, color=~Design), size=1.5, alpha=0.8, stat = "unique") +
+    ggplot2::geom_line(dat, mapping=ggplot2::aes(x=.data$N, y=.data$rmse, group=.data$Design, color=.data$Design), alpha=0.8, stat = "unique") +
+    ggplot2::geom_point(dat, mapping=ggplot2::aes(x=.data$N, y=.data$rmse, group=.data$Design, color=.data$Design), size=1.5, alpha=0.8, stat = "unique") +
     ggplot2::scale_colour_manual(values = c24, breaks=unique(dat$Design))
 
   p <- p + ggplot2::theme(
