@@ -13,8 +13,8 @@ pkg.env$dill <- NULL
 #'
 #' @description This function initializes the 'python' environment for the package.
 #'
-#' @param py_ver a string that gives the 'python' version to be installed. If `py_ver = NULL`, the default 'python'
-#'     version '3.9.13' will be installed.
+#' @param py_ver a string that gives the 'python' version to be installed. Supported versions are 3.10, 3.11, and 3.12.
+#'    If `py_ver = NULL`, the default 'python' version '3.10' will be installed.
 #' @param dgpsi_ver a string that gives the 'python' version of 'dgpsi' to be used. If `dgpsi_ver = NULL`,
 #' * the latest 'python' version of 'dgpsi' will be used, if the package is installed from CRAN;
 #' * the development 'python' version of 'dgpsi' will be used, if the package is installed from GitHub.
@@ -40,22 +40,23 @@ pkg.env$dill <- NULL
 #' @md
 #' @export
 init_py <- function(py_ver = NULL, dgpsi_ver = NULL, reinstall = FALSE, uninstall = FALSE, verb = TRUE) {
-  if ( is.null(py_ver) ) py_ver <- '3.9.13'
+  if ( is.null(py_ver) ) py_ver <- '3.10'
   if ( is.null(dgpsi_ver) ) {
     ##For devel version
-    #dgpsi_ver <- c('cython>=0.29.30', 'dill>=0.3.2, <=0.3.5.1', 'jupyter>=1.0.0', 'matplotlib-base>=3.2.1', 'numba >=0.51.2',
-    #               'numpy >=1.18.2', 'pathos ==0.2.9', 'multiprocess ==0.70.13', 'psutil >=5.8.0', 'pybind11 >=2.10.0', 'pythran >=0.11.0',
-    #               'scikit-build >=0.15.0', 'scikit-learn >=0.22.0', 'scipy >=1.4.1', 'tqdm >=4.50.2', 'tabulate >=0.8.7', 'faiss-cpu >=1.7.4')
-    #env_name <- 'dgp_si_R_2_4_0_9000'
+    #dgpsi_ver <- c('dill>=0.3.2', 'matplotlib-base>=3.2.1', 'numba >=0.51.2',
+    #               'numpy >=1.18.2', 'pathos >=0.2.9', 'multiprocess >=0.70.13', 'psutil >=5.8.0',
+    #               'scikit-learn >=0.22.0', 'scipy >=1.4.1', 'tqdm >=4.50.2', 'tabulate >=0.8.7', 'faiss-cpu >=1.7.4')
+    #env_name <- 'dgp_si_R_2_5_0_9000'
     ##For release version
-    dgpsi_ver <- 'dgpsi==2.5.0'
-    env_name <- 'dgp_si_R_2_5_0'
+    dgpsi_ver <- 'dgpsi==2.6.0'
+    env_name <- 'dgp_si_R_2_6_0'
   } else {
     env_name <- paste('dgp_si_R_', gsub(".", "_", dgpsi_ver,fixed=TRUE), sep = "")
     dgpsi_ver <- paste('dgpsi==', dgpsi_ver, sep = "")
   }
   #Check if there is any conda binary installed, if not, request to install it.
   #restart <- FALSE
+  Sys.setenv(CONDA_PLUGINS_AUTO_ACCEPT_TOS = "yes")
   if (is.null(tryCatch(reticulate::conda_binary(), error = function(e) NULL))){
     ans <- readline(prompt="I am unable to find a conda binary. Do you want me to install it for you? (Y/N) ")
     #If the user would like to have the conda binary to be installed
@@ -148,19 +149,26 @@ init_py <- function(py_ver = NULL, dgpsi_ver = NULL, reinstall = FALSE, uninstal
 
 install_dgpsi <- function(env_name, py_ver, conda_path, dgpsi_ver, reinsatll = FALSE) {
   if (!reinsatll) message(sprintf("Setting up the Python environment for %s ...\n", dgpsi_ver))
-  if (!reinsatll) reticulate::conda_create(envname = env_name, python_version = py_ver, conda = conda_path)
+  if (!reinsatll) reticulate::conda_create(envname = env_name, python_version = py_ver, conda = conda_path, forge = TRUE, additional_create_args = c('--strict-channel-priority'))
   if (reinsatll) {
     message("Re-installing the required Python packages ...")
   } else {
     message("Installing the required Python packages ...")
   }
   if (Sys.info()[["sysname"]] == "Darwin" & Sys.info()[["machine"]] == "arm64"){
-    reticulate::conda_install(envname = env_name, packages = c(dgpsi_ver, '"libblas=*=*accelerate"') , conda = conda_path)
-  } else if (grepl("Intel",benchmarkme::get_cpu()$model_name)){
-    reticulate::conda_install(envname = env_name, packages = c(dgpsi_ver, '"libblas=*=*mkl"') , conda = conda_path)
+    macos_version <- system("sw_vers -productVersion", intern = TRUE)
+    version_nums <- as.numeric(strsplit(macos_version, "\\.")[[1]])
+    current_version <- version_nums[1] + version_nums[2] / 10
+    if (current_version>=13.3){
+      reticulate::conda_install(envname = env_name, packages = c(dgpsi_ver, '"libblas=*=*newaccelerate"') , conda = conda_path, forge = TRUE, additional_install_args = c('--strict-channel-priority'))
+    } else {
+      reticulate::conda_install(envname = env_name, packages = c(dgpsi_ver, '"libblas=*=*accelerate"') , conda = conda_path, forge = TRUE, additional_install_args = c('--strict-channel-priority'))
+    }
+  } else if ( isTRUE(grepl("Intel",benchmarkme::get_cpu()$model_name)) ){
+    reticulate::conda_install(envname = env_name, packages = c(dgpsi_ver, '"libblas=*=*mkl"') , conda = conda_path, forge = TRUE, additional_install_args = c('--strict-channel-priority') )
     reticulate::conda_install(envname = env_name, packages = c('icc_rt') , channel = c("numba"), conda = conda_path)
   } else {
-    reticulate::conda_install(envname = env_name, packages = c(dgpsi_ver) , conda = conda_path)
+    reticulate::conda_install(envname = env_name, packages = c(dgpsi_ver) , conda = conda_path, forge = TRUE, additional_install_args = c('--strict-channel-priority'))
   }
   if (grepl('9000',env_name)) {
     if (reinsatll) {
