@@ -1,10 +1,6 @@
 #' @title Locate the next design point for a (D)GP emulator or a bundle of (D)GP emulators using MICE
 #'
-#' @description
-#'
-#' `r new_badge("updated")`
-#'
-#' This function searches from a candidate set to locate the next design point(s) to be added to a (D)GP emulator
+#' @description This function searches from a candidate set to locate the next design point(s) to be added to a (D)GP emulator
 #'     or a bundle of (D)GP emulators using the Mutual Information for Computer Experiments (MICE), see the reference below.
 #'
 #' @param object can be one of the following:
@@ -19,16 +15,16 @@
 #'     This argument is used only when `x_cand` is `NULL`. Defaults to `200`.
 #' @param batch_size an integer that gives the number of design points to be chosen.
 #'     Defaults to `1`.
-#' @param M `r new_badge("new")` the size of the conditioning set for the Vecchia approximation in the criterion calculation. This argument is only used if the emulator `object`
+#' @param M the size of the conditioning set for the Vecchia approximation in the criterion calculation. This argument is only used if the emulator `object`
 #'     was constructed under the Vecchia approximation. Defaults to `50`.
 #' @param nugget_s the value of the smoothing nugget term used by MICE. Defaults to `1e-6`.
 #' @param workers  the number of processes to be used for the criterion calculation. If set to `NULL`,
 #'     the number of processes is set to `max physical cores available %/% 2`. Defaults to `1`.
-#' @param limits `r new_badge("new")` a two-column matrix that gives the ranges of each input dimension, or a vector of length two if there is only one input dimension.
+#' @param limits a two-column matrix that gives the ranges of each input dimension, or a vector of length two if there is only one input dimension.
 #'     If a vector is provided, it will be converted to a two-column row matrix. The rows of the matrix correspond to input dimensions, and its
 #'     first and second columns correspond to the minimum and maximum values of the input dimensions. This
 #'     argument is only used when `x_cand = NULL`. Defaults to `NULL`.
-#' @param int `r new_badge("new")` a bool or a vector of bools that indicates if an input dimension is an integer type. If a single bool is given, it will be applied to
+#' @param int a bool or a vector of bools that indicates if an input dimension is an integer type. If a single bool is given, it will be applied to
 #'     all input dimensions. If a vector is provided, it should have a length equal to the input dimensions and will be applied to individual
 #'     input dimensions. This argument is only used when `x_cand = NULL`. Defaults to `FALSE`.
 #' @param aggregate an R function that aggregates scores of the MICE across different output dimensions (if `object` is an instance
@@ -129,6 +125,7 @@ mice.gp <- function(object, x_cand = NULL, n_cand = 200, batch_size = 1, M = 50,
   }
   #check class
   if ( !inherits(object,"gp") ) stop("'object' must be an instance of the 'gp' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   training_input <- object$data$X
   training_output <- object$data$Y
   n_dim_X <- ncol(training_input)
@@ -220,6 +217,7 @@ mice.dgp <- function(object, x_cand = NULL, n_cand = 200, batch_size = 1, M = 50
   }
   #check class
   if ( !inherits(object,"dgp") ) stop("'object' must be an instance of the 'dgp' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   #if ( object$constructor_obj$all_layer[[object$constructor_obj$n_layer]][[1]]$type == 'likelihood' ){
   #  stop("The function is only applicable to DGP emulators without likelihood layers.", call. = FALSE)
   #}
@@ -363,6 +361,7 @@ mice.bundle <- function(object, x_cand = NULL, n_cand = 200, batch_size = 1, M =
   }
   #check class
   if ( !inherits(object,"bundle") ) stop("'object' must be an instance of the 'bundle' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$emulator1$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulators in the bundle or, if the bundle was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   #check no of emulators
   n_emulators <- length(object)
   if ( "data" %in% names(object) ) n_emulators <- n_emulators - 1
