@@ -1,41 +1,19 @@
 #' @title Prediction from GP, DGP, or linked (D)GP emulators
 #'
-#' @description
-#'
-#' `r new_badge("updated")`
-#'
-#' This function implements prediction from GP, DGP, or linked (D)GP emulators.
+#' @description This function implements prediction from GP, DGP, or linked (D)GP emulators.
 #'
 #' @param object an instance of the `gp`, `dgp`, or `lgp` class.
 #' @param x the testing input data:
 #' * if `object` is an instance of the `gp` or `dgp` class, `x` is a matrix where each row is an input testing data point and each column is an input dimension.
-#' * `r lifecycle::badge("deprecated")` if `object` is an instance of the `lgp` class created by [lgp()] without specifying argument `struc` in data frame form, `x` can be either a matrix or a list:
-#'    - if `x` is a matrix, its rows are treated as instances of the `Global` inputs. In this case, it is assumed that the only global input to the system is the input to the
-#'      emulators in the first layer and there is no global input to emulators in other layers.
-#'    - if `x` is a list, it should have *L* (the number of layers in an emulator system) elements. The first element
-#'      is a matrix that represents the global testing input data that feed into the emulators in the first layer of the system. The
-#'      remaining *L-1* elements are *L-1* sub-lists, each of which contains a number (the same number of emulators in
-#'      the corresponding layer) of matrices (rows being testing input data points and columns being input dimensions) that represent the
-#'      global testing input data to the emulators in the corresponding layer. The matrices must be placed in the sub-lists based on how
-#'      their corresponding emulators are placed in `struc` argument of [lgp()]. If there is no global input data to a certain emulator,
-#'      set `NULL` in the corresponding sub-list of `x`.
-#'
-#'    **This option for linked (D)GP emulators is deprecated and will be removed in the next release.**
-#' * `r new_badge("new")` If `object` is an instance of the `lgp` class created by [lgp()] with argument `struc` in data frame form,
-#'   `x` must be a matrix representing the global input, where each row corresponds to a test data point and each column represents a global input dimension.
+#' * if `object` is an instance of the `lgp` class, `x` must be a matrix representing the global input, where each row corresponds to a test data point and each column represents a global input dimension.
 #'   The column indices in `x` must align with the indices specified in the `From_Output` column of the `struc` data frame (used in [lgp()]),
 #'   corresponding to rows where the `From_Emulator` column is `"Global"`.
 #' @param method `r new_badge("updated")` the prediction approach to use: either the mean-variance approach (`"mean_var"`) or the sampling approach (`"sampling"`).
 #'     The mean-variance approach returns the means and variances of the predictive distributions, while the sampling approach generates samples from predictive distributions
-#'     using the derived means and variances. For DGP emulators with a categorical likelihood (`likelihood = "Categorical"` in [dgp()]), `method` is only applicable
-#'     when `full_layer = TRUE`. In this case, the sampling approach generates samples from the GP nodes in all hidden layers using the derived means and variances,
-#'     and subsequently propagates these samples through the categorical likelihood. By default, the method is set to `"sampling"` for DGP emulators with Poisson, Negative Binomial, and
-#'     Categorical likelihoods, and to `"mean_var"` otherwise.
-#' @param mode `r new_badge("new")` whether to predict the classes (`"label"`) or probabilities (`"proba"`) of different classes when `object` is a DGP emulator with a categorical likelihood.
-#'      Defaults to `"label"`.
+#'     using the derived means and variances. Defaults to `"mean_var"`.
 #' @param full_layer a bool indicating whether to output the predictions of all layers. Defaults to `FALSE`. Only used when `object` is a DGP or a linked (D)GP emulator.
 #' @param sample_size the number of samples to draw for each given imputation if `method = "sampling"`. Defaults to `50`.
-#' @param M `r new_badge("new")` the size of the conditioning set for the Vecchia approximation in the emulator prediction. Defaults to `50`. This argument is only used if the emulator `object`
+#' @param M the size of the conditioning set for the Vecchia approximation in the emulator prediction. Defaults to `50`. This argument is only used if the emulator `object`
 #'     was constructed under the Vecchia approximation.
 #' @param cores the number of processes to be used for prediction. If set to `NULL`, the number of processes is set to `max physical cores available %/% 2`. Defaults to `1`.
 #' @param chunks the number of chunks that the testing input matrix `x` will be divided into for multi-cores to work on.
@@ -49,51 +27,27 @@
 #'      corresponding to testing positions (i.e., rows of `x`).
 #'   2. if `method = "sampling"`: an updated `object` is returned with an additional slot called `results` that contains a matrix whose rows correspond
 #'      to testing positions and columns correspond to `sample_size` number of samples drawn from the predictive distribution of GP.
-#' * If `object` is an instance of the `dgp` class:
+#' * `r new_badge("updated")` If `object` is an instance of the `dgp` class:
 #'   1. if `method = "mean_var"` and  `full_layer = FALSE`: an updated `object` is returned with an additional slot called `results` that contains two
 #'      matrices named `mean` for the predictive means and `var` for the predictive variances respectively. Each matrix has its rows corresponding to testing
-#'      positions and columns corresponding to DGP global output dimensions (i.e., the number of GP/likelihood nodes in the final layer).
+#'      positions and columns corresponding to DGP global output dimensions (i.e., the number of GP/likelihood nodes in the final layer). If the likelihood node
+#'      is categorical, the matrices contain the predictive means and variances of the class probabilities, with columns corresponding to different classes.
 #'   2. if `method = "mean_var"` and  `full_layer = TRUE`: an updated `object` is returned with an additional slot called `results` that contains two
 #'      sub-lists named `mean` for the predictive means and `var` for the predictive variances respectively. Each sub-list contains *L* (i.e., the number of layers)
 #'      matrices named `layer1, layer2,..., layerL`. Each matrix has its rows corresponding to testing positions and columns corresponding to
-#'      output dimensions (i.e., the number of GP/likelihood nodes from the associated layer).
+#'      output dimensions (i.e., the number of GP/likelihood nodes from the associated layer). If the likelihood node is categorical, the matrices named `layerL`
+#'      in both `mean` and `var` contain the predictive means and variances of the class probabilities, respectively, with columns corresponding to different classes.
 #'   3. if `method = "sampling"` and  `full_layer = FALSE`: an updated `object` is returned with an additional slot called `results` that contains *D* (i.e., the number
-#'      of GP/likelihood nodes in the final layer) matrices named `output1, output2,..., outputD`. Each matrix has its rows corresponding to testing positions and
-#'      columns corresponding to samples of size: `B * sample_size`, where `B` is the number of imputations specified in [dgp()].
+#'      of GP/likelihood nodes in the final layer) matrices named `output1, output2,..., outputD`. If the likelihood node in the final layer is categorical, `results`
+#'      contains *D* matrices (where *D* is the number of classes) of sampled class probabilities, each named according to its corresponding class label. Each matrix in `results`
+#'      has its rows corresponding to testing positions and columns corresponding to samples of size: `B * sample_size`, where `B` is the number of imputations specified in [dgp()].
 #'   4. if `method = "sampling"` and  `full_layer = TRUE`: an updated `object` is returned with an additional slot called `results` that contains *L* (i.e., the number
 #'      of layers) sub-lists named `layer1, layer2,..., layerL`. Each sub-list represents samples drawn from the GP/likelihood nodes in the corresponding layer,
-#'      and contains *D* (i.e., the number of GP/likelihood nodes in the corresponding layer) matrices named `output1, output2,..., outputD`. Each matrix gives samples
-#'      of the output from one of *D* GP/likelihood nodes, and has its rows corresponding to testing positions and columns corresponding to samples
+#'      and contains *D* (i.e., the number of GP/likelihood nodes in the corresponding layer) matrices named `output1, output2,..., outputD`. If the likelihood node in the final
+#'      layer is categorical, `layerL` contains *D* matrices (where *D* is the number of classes) of sampled class probabilities, each named according to its corresponding class
+#'      label. Each matrix has its rows corresponding to testing positions and columns corresponding to samples
 #'      of size: `B * sample_size`, where `B` is the number of imputations specified in [dgp()].
-#' * `r new_badge("new")` If `object` is an instance of the `dgp` class with a categorical likelihood:
-#'   1. if `full_layer = FALSE` and `mode = "label"`: an updated `object` is returned with an additional slot called `results` that contains one matrix named `label`.
-#'      The matrix has rows corresponding to testing positions and columns corresponding to sample labels of size: `B * sample_size`, where `B` is the number
-#'      of imputations specified in [dgp()].
-#'   2. if `full_layer = FALSE` and `mode = "proba"`, an updated `object` is returned with an additional slot called `results`. This slot contains *D* matrices (where
-#'      *D* is the number of classes in the training output), where each matrix gives probability samples for the corresponding class with its rows corresponding to testing
-#'      positions and columns containing probabilities. The number of columns of each matrix is `B * sample_size`, where `B` is the number of imputations
-#'      specified in the [dgp()] function.
-#'   3. if `method = "mean_var"` and `full_layer = TRUE`: an updated `object` is returned with an additional slot called `results` that contains *L* (i.e., the number
-#'      of layers) sub-lists named `layer1, layer2,..., layerL`. Each of first `L-1` sub-lists contains two matrices named `mean` for the predictive means and `var`
-#'      for the predictive variances of the GP nodes in the associated layer. Rows of each matrix correspond to testing positions.
-#'      - when `mode = "label"`, the sub-list `LayerL` contains one matrix named `label`. The matrix has its rows corresponding to testing positions and columns
-#'        corresponding to label samples of size: `B * sample_size`. `B` is the number of imputations specified in [dgp()].
-#'      - when `mode = "proba"`, the sub-list `LayerL` contains *D* matrices (where *D* is the number of classes in the training output), where each matrix gives probability
-#'        samples for the corresponding class with its rows corresponding to testing positions and columns containing probabilities. The number of columns of each matrix
-#'        is `B * sample_size`. `B` is the number of imputations specified in [dgp()].
-#'   4. if `method = "sampling"` and `full_layer = TRUE`: an updated `object` is returned with an additional slot called `results` that contains *L* (i.e., the number
-#'      of layers) sub-lists named `layer1, layer2,..., layerL`. Each of first `L-1` sub-lists represents samples drawn from the GP nodes in the
-#'      corresponding layer, and contains *D* (i.e., the number of GP nodes in the corresponding layer) matrices named `output1, output2,..., outputD`. Each matrix
-#'      gives samples of the output from one of *D* GP nodes, and has its rows corresponding to testing positions and columns corresponding to samples
-#'      of size: `B * sample_size`.
-#'      - when `mode = "label"`, the sub-list `LayerL` contains one matrix named `label`. The matrix has its rows corresponding to testing positions and columns
-#'        corresponding to label samples of size: `B * sample_size`.
-#'      - when `mode = "proba"`, the sub-list `LayerL` contains *D* matrices (where *D* is the number of classes in the training output), where each matrix gives probability
-#'        samples for the corresponding class with its rows corresponding to testing positions and columns containing probabilities. The number of columns of each matrix
-#'        is `B * sample_size`.
-#'
-#'      `B` is the number of imputations specified in [dgp()].
-#' * `r new_badge("updated")` If `object` is an instance of the `lgp` class:
+#' * If `object` is an instance of the `lgp` class:
 #'   1. if `method = "mean_var"` and  `full_layer = FALSE`: an updated `object` is returned with an additional slot called `results` that
 #'      contains two sub-lists named `mean` for the predictive means and `var` for the predictive variances respectively. Each sub-list
 #'      contains *K* (same number of emulators in the final layer of the system) matrices named using the `ID`s of the corresponding emulators in the final layer.
@@ -117,11 +71,8 @@
 #'      the output dimensions of the GP/DGP emulator. Each matrix has its rows corresponding to testing positions and columns corresponding to
 #'      samples of size: `B * sample_size`, where `B` is the number of imputations specified in [lgp()].
 #'
-#'   If `object` is an instance of the `lgp` class created by [lgp()] without specifying the `struc` argument in data frame form, the `ID`s, that are used as names of sub-lists or
-#'   matrices within `results`, will be replaced by `emulator1`, `emulator2`, and so on.
-#'
 #' The `results` slot will also include:
-#' * `r new_badge("new")` the value of `M`, which represents the size of the conditioning set for the Vecchia approximation, if used, in the emulator prediction.
+#' * the value of `M`, which represents the size of the conditioning set for the Vecchia approximation, if used, in the emulator prediction.
 #' * the value of `sample_size` if `method = "sampling"`.
 #'
 #' @details See further examples and tutorials at <`r get_docs_url()`>.
@@ -138,12 +89,13 @@ NULL
 #' @rdname predict
 #' @method predict dgp
 #' @export
-predict.dgp <- function(object, x, method = NULL, mode = 'label', full_layer = FALSE, sample_size = 50, M = 50, cores = 1, chunks = NULL, ...) {
+predict.dgp <- function(object, x, method = "mean_var", full_layer = FALSE, sample_size = 50, M = 50, cores = 1, chunks = NULL, ...) {
   if ( is.null(pkg.env$dgpsi) ) {
     init_py(verb = F)
     if (pkg.env$restart) return(invisible(NULL))
   }
   if ( !inherits(object,"dgp") ) stop("'object' must be an instance of the 'dgp' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   if ( !is.matrix(x)&!is.vector(x) ) stop("'x' must be a vector or a matrix.", call. = FALSE)
   if ( is.vector(x) ) {
     if ( ncol(object$data$X)!=1 ){
@@ -158,36 +110,14 @@ predict.dgp <- function(object, x, method = NULL, mode = 'label', full_layer = F
   final_node <- object$specs[[paste('layer', L, sep="")]][['node1']]
   if ("type" %in% names(final_node) && final_node$type == "Categorical") {
     is.categorical <- TRUE
-    is.Poisson <- FALSE
-    is.NegBin <- FALSE
     encoder <- object$constructor_obj$all_layer[[L]][[1]]$class_encoder
     index_to_label <- as.character(encoder$classes_)
-  } else if ("type" %in% names(final_node) && final_node$type == "Poisson") {
-    is.categorical <- FALSE
-    is.Poisson <- TRUE
-    is.NegBin <- FALSE
-  } else if ("type" %in% names(final_node) && final_node$type == "NegBin") {
-    is.categorical <- FALSE
-    is.Poisson <- FALSE
-    is.NegBin <- TRUE
   } else {
     is.categorical <- FALSE
-    is.Poisson <- FALSE
-    is.NegBin <- FALSE
   }
 
-  if ( is.null(method) ){
-    if (is.categorical|is.Poisson|is.NegBin) {
-      method = 'sampling'
-    } else {
-      method = 'mean_var'
-    }
-  } else {
-    if ( method!='mean_var' & method!='sampling' ) stop("'method' can only be either 'mean_var' or 'sampling'.", call. = FALSE)
-    if ( method=='mean_var' && is.categorical && !full_layer){
-      method = 'sampling'
-    }
-  }
+
+  if ( method!='mean_var' & method!='sampling' ) stop("'method' can only be either 'mean_var' or 'sampling'.", call. = FALSE)
 
   sample_size <- as.integer(sample_size)
   M <- as.integer(M)
@@ -205,61 +135,51 @@ predict.dgp <- function(object, x, method = NULL, mode = 'label', full_layer = F
   rep <- rep_x[[2]] + 1
 
   if ( identical(cores,as.integer(1)) ){
-    if (is.categorical) {
-      res <- object$emulator_obj$classify(reticulate::np_array(x_unique), mode, method, full_layer, sample_size, M)
-    } else {
-      res <- object$emulator_obj$predict(x_unique, method, full_layer, sample_size, M)
-    }
+    res <- object$emulator_obj$predict(x_unique, method, full_layer, sample_size, M)
   } else {
-    if (is.categorical) {
-      res <- object$emulator_obj$pclassify(reticulate::np_array(x_unique), mode, method, full_layer, sample_size, M, chunks, cores)
-    } else {
-      res <- object$emulator_obj$ppredict(x_unique, method, full_layer, sample_size, M, chunks, cores)
-    }
+    res <- object$emulator_obj$ppredict(x_unique, method, full_layer, sample_size, M, chunks, cores)
   }
 
   if (method == 'mean_var'){
     if (full_layer) {
+      named_res <- list("mean" = res[[1]], "var" = res[[2]])
+      LL <- length(named_res$mean)
+      for (l in 1:LL) {
+        named_res$mean[[l]] <- named_res$mean[[l]][rep,,drop=F]
+        named_res$var[[l]] <- named_res$var[[l]][rep,,drop=F]
+        names(named_res$mean)[l] <- paste('layer', l, sep="")
+        names(named_res$var)[l] <- paste('layer', l, sep="")
+      }
       if (is.categorical){
-        named_res <- vector('list', L)
-        for (l in 1:(L-1)) {
-          names(named_res)[l] <- paste('layer', l, sep="")
-          named_res[[l]][[1]] <- res[[1]][[l]][rep,,drop=F]
-          named_res[[l]][[2]] <- res[[2]][[l]][rep,,drop=F]
-          names(named_res[[l]])[1] <- 'mean'
-          names(named_res[[l]])[2] <- 'var'
-        }
-        names(named_res)[L] <- paste('layer', L, sep="")
-        for (k in 1:length(res[[3]])) {
-          named_res[[L]][[k]] <- res[[3]][[k]][rep,,drop=F]
-          if ( mode == 'label' ){
-            names(named_res[[L]])[k] <- 'label'
-          } else {
-            names(named_res[[L]])[k] <- index_to_label[k]
-          }
-        }
-      } else {
-        named_res <- list("mean" = res[[1]], "var" = res[[2]])
-        for (l in 1:length(named_res$mean)) {
-          named_res$mean[[l]] <- named_res$mean[[l]][rep,,drop=F]
-          named_res$var[[l]] <- named_res$var[[l]][rep,,drop=F]
-          names(named_res$mean)[l] <- paste('layer', l, sep="")
-          names(named_res$var)[l] <- paste('layer', l, sep="")
+        if (length(index_to_label)==2){
+          named_res$mean[[LL]] <- cbind(1-named_res$mean[[LL]], named_res$mean[[LL]])
+          named_res$var[[LL]] <- cbind(named_res$var[[LL]], named_res$var[[LL]])
         }
+        colnames(named_res$mean[[LL]]) <- index_to_label
+        colnames(named_res$var[[LL]]) <- index_to_label
       }
     } else {
       named_res <- list("mean" = res[[1]][rep,,drop=F], "var" = res[[2]][rep,,drop=F])
+      if (is.categorical){
+        if (length(index_to_label)==2){
+          named_res$mean <- cbind(1-named_res$mean, named_res$mean)
+          named_res$var <- cbind(named_res$var, named_res$var)
+        }
+        colnames(named_res$mean) <- index_to_label
+        colnames(named_res$var) <- index_to_label
+      }
     }
   } else if (method == 'sampling') {
     if (full_layer) {
       named_res <- res
-      for (l in 1:length(named_res)) {
+      LL <- length(named_res)
+      for (l in 1:LL) {
         names(named_res)[l] <- paste('layer', l, sep="")
         for (k in 1:length(named_res[[l]])) {
           named_res[[l]][[k]] <- named_res[[l]][[k]][rep,,drop=F]
-          if ( l == length(named_res) && is.categorical ) {
-            if ( mode == 'label' ){
-              names(named_res[[l]])[k] <- 'label'
+          if ( l == LL && is.categorical ) {
+            if (length(index_to_label)==2){
+              names(named_res[[l]])[k] <- index_to_label[k+1]
             } else {
               names(named_res[[l]])[k] <- index_to_label[k]
             }
@@ -268,13 +188,17 @@ predict.dgp <- function(object, x, method = NULL, mode = 'label', full_layer = F
           }
         }
       }
+      if (is.categorical && length(index_to_label)==2){
+        named_res[[LL]][[2]] <- 1 - named_res[[LL]][[1]]
+        names(named_res[[LL]])[2] <- index_to_label[1]
+      }
     } else {
       named_res <- res
       for (l in 1:length(named_res)) {
         named_res[[l]] <- named_res[[l]][rep,,drop=F]
         if ( is.categorical ) {
-          if ( mode == 'label' ){
-            names(named_res)[l] <- 'label'
+          if (length(index_to_label)==2){
+            names(named_res)[l] <- index_to_label[l+1]
           } else {
             names(named_res)[l] <- index_to_label[l]
           }
@@ -282,12 +206,16 @@ predict.dgp <- function(object, x, method = NULL, mode = 'label', full_layer = F
           names(named_res)[l] <- paste('output', l, sep="")
         }
       }
+      if (is.categorical && length(index_to_label)==2){
+        named_res[[2]] <- 1 - named_res[[1]]
+        names(named_res)[2] <- index_to_label[1]
+      }
     }
   }
 
   object$results <- named_res
   object$results[["M"]] <- M
-  if (method == "sampling" | is.categorical){
+  if (method == "sampling"){
     object$results[["sample_size"]] <- sample_size
   }
   return(object)
@@ -297,16 +225,17 @@ predict.dgp <- function(object, x, method = NULL, mode = 'label', full_layer = F
 #' @rdname predict
 #' @method predict lgp
 #' @export
-predict.lgp <- function(object, x, method = NULL, full_layer = FALSE, sample_size = 50, M = 50, cores = 1, chunks = NULL, ...) {
+predict.lgp <- function(object, x, method = "mean_var", full_layer = FALSE, sample_size = 50, M = 50, cores = 1, chunks = NULL, ...) {
   if ( is.null(pkg.env$dgpsi) ) {
     init_py(verb = F)
     if (pkg.env$restart) return(invisible(NULL))
   }
   if ( !inherits(object,"lgp") ) stop("'object' must be an instance of the 'lgp' class.", call. = FALSE)
-  if ( "metadata" %in% names(object$specs) ){
+
     if ( !("emulator_obj" %in% names(object)) ){
       stop("'object' is not activated for predictions. Please set `activate = TRUE` in `lgp()` to activate the emulator.", call. = FALSE)
     }
+    if ( reticulate::py_is_null_xptr(object$emulator_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
     if ( !is.matrix(x)&!is.vector(x) ) stop("'x' must be a vector or a matrix.", call. = FALSE)
     x <- unname(x)
     global_dim <- unique(subset(object$specs$struc, object$specs$struc[["From_Emulator"]] == "Global")$From_Output)
@@ -337,118 +266,9 @@ predict.lgp <- function(object, x, method = NULL, full_layer = FALSE, sample_siz
       x_list[[l]] <- layer_matrices
     }
     x <- x_list
-  } else {
-    lifecycle::deprecate_warn(
-      when = "2.5.0",
-      what = I("The `object` created by `lgp()` without specifying `struc` as a data frame"),
-      details = c(
-        i = "Support for `object` structures created without `struc` specified as a data frame will be removed in the next release.",
-        i = "To ensure compatibility with future versions, please recreate the `object` by calling the updated `lgp()` with `struc` provided as a data frame."
-      )
-    )
-    # To be deprecated
-    if ( !is.list(x) ) {
-      if ( !is.matrix(x)&!is.vector(x) ) stop("'x' must be a vector or a matrix.", call. = FALSE)
-      x <- unname(x)
-      if ( is.vector(x) ) {
-        is.1d <- TRUE
-        for (item in object$constructor_obj[[1]]){
-          if (item$type == 'gp'){
-            if (length(item$structure$input_dim) != 1){
-              is.1d <- FALSE
-              break
-            }
-          } else {
-            for (ker in item$structure[[1]]){
-              if (length(ker$input_dim) != 1){
-                is.1d <- FALSE
-                break
-              }
-            }
-            if (isFALSE(is.1d)) break
-          }
-        }
-        if ( is.1d ){
-          x <- as.matrix(x)
-        } else {
-          x <- matrix(x, nrow = 1)
-        }
-      }
-    } else {
-      for ( l in 1:length(x) ){
-        if ( l==1 ){
-          if ( !is.matrix(x[[l]])&!is.vector(x[[l]]) ) {
-            stop("The first element of 'x' must be a vector or a matrix.", call. = FALSE)
-          } else {
-            x[[l]] <- unname(x[[l]])
-            if ( is.vector(x[[l]]) ) {
-              is.1d <- TRUE
-              for (item in object$constructor_obj[[l]]){
-                if (item$type == 'gp'){
-                  if (length(item$structure$input_dim) != 1){
-                    is.1d <- FALSE
-                    break
-                  }
-                } else {
-                  for (ker in item$structure[[1]]){
-                    if (length(ker$input_dim) != 1){
-                      is.1d <- FALSE
-                      break
-                    }
-                  }
-                  if (isFALSE(is.1d)) break
-                }
-              }
-              if ( is.1d ){
-                x[[l]] <- as.matrix(x[[l]])
-              } else {
-                x[[l]] <- matrix(x[[l]], nrow = 1)
-              }
-            }
-            nrow_x <- nrow(x[[l]])
-          }
-        } else {
-          for ( k in 1:length(x[[l]]) ){
-            if ( !is.matrix(x[[l]][[k]])&!is.null(x[[l]][[k]])&!is.vector(x[[l]][[k]]) ) stop(sprintf("The element %i in the sublist %i of 'x' must be a vector, a matrix, or 'NULL'.", k, l), call. = FALSE)
-            if ( is.matrix(x[[l]][[k]])|is.vector(x[[l]][[k]]) ){
-              x[[l]][[k]] <- unname(x[[l]][[k]])
-              if (is.vector(x[[l]][[k]])) {
-                is.1d <- TRUE
-                item_cont <- object$constructor_obj[[l]][[k]]
-                if (item_cont$type == 'gp'){
-                  if (length(item_cont$structure$input_dim) != 1){
-                    is.1d <- FALSE
-                  }
-                } else {
-                  for (ker in item_cont$structure[[1]]){
-                    if (length(ker$input_dim) != 1){
-                      is.1d <- FALSE
-                      break
-                    }
-                  }
-                }
-                if ( is.1d ){
-                  x[[l]][[k]] <- as.matrix(x[[l]][[k]])
-                } else {
-                  x[[l]][[k]] <- matrix(x[[l]][[k]], nrow = 1)
-                }
-              }
-              if ( nrow(x[[l]][[k]])!=nrow_x ) {
-                stop(sprintf("The element %i in the sublist %i of 'x' has inconsistent number of data points with the first element of 'x'.", k, l), call. = FALSE)
-              }
-            }
-          }
-        }
-      }
-    }
-  }
 
 
-  if ( is.null(method) ){
-    method = 'mean_var'
-  } else {
-    if ( method!='mean_var' & method!='sampling' ) stop("'method' can only be either 'mean_var' or 'sampling'.", call. = FALSE)
-  }
+  if ( method!='mean_var' & method!='sampling' ) stop("'method' can only be either 'mean_var' or 'sampling'.", call. = FALSE)
 
   sample_size <- as.integer(sample_size)
   M <- as.integer(M)
@@ -474,28 +294,18 @@ predict.lgp <- function(object, x, method = NULL, full_layer = FALSE, sample_siz
         names(named_res$mean)[l] <- paste('layer', l, sep="")
         names(named_res$var)[l] <- paste('layer', l, sep="")
         for (k in 1:length(named_res$mean[[l]])) {
-          if ( "metadata" %in% names(object$specs) ){
             emulator_id <- subset(object$specs$metadata, object$specs$metadata[["Layer"]] == l & object$specs$metadata[["Pos_in_Layer"]] == k)$Emulator
             names(named_res$mean[[l]])[k] <- emulator_id
             names(named_res$var[[l]])[k] <- emulator_id
-          } else {
-            names(named_res$mean[[l]])[k] <- paste('emulator', k, sep="")
-            names(named_res$var[[l]])[k] <- paste('emulator', k, sep="")
-          }
         }
       }
     } else {
       named_res <- list("mean" = res[[1]], "var" = res[[2]])
-      if ( "metadata" %in% names(object$specs) ) final_layer <- max(object$specs$metadata$Layer)
+      final_layer <- max(object$specs$metadata$Layer)
       for (k in 1:length(named_res$mean)) {
-        if ( "metadata" %in% names(object$specs) ){
           emulator_id <- subset(object$specs$metadata, object$specs$metadata[["Layer"]] == final_layer & object$specs$metadata[["Pos_in_Layer"]] == k)$Emulator
           names(named_res$mean)[k] <- emulator_id
           names(named_res$var)[k] <- emulator_id
-        } else {
-          names(named_res$mean)[k] <- paste('emulator', k, sep="")
-          names(named_res$var)[k] <- paste('emulator', k, sep="")
-        }
       }
     }
   } else if (method == 'sampling') {
@@ -504,11 +314,7 @@ predict.lgp <- function(object, x, method = NULL, full_layer = FALSE, sample_siz
       for (l in 1:length(res)) {
         name1 <- paste('layer', l, sep="")
         for (k in 1:length(res[[l]])){
-          if ( "metadata" %in% names(object$specs) ){
             name2 <- subset(object$specs$metadata, object$specs$metadata[["Layer"]] == l & object$specs$metadata[["Pos_in_Layer"]] == k)$Emulator
-          } else {
-            name2 <- paste('emulator', k, sep="")
-          }
           for (n in 1:dim(res[[l]][[k]])[1]) {
             name3 <- paste('output', n, sep="")
             named_res[[name1]][[name2]][[name3]] <- res[[l]][[k]][n,,]
@@ -517,13 +323,9 @@ predict.lgp <- function(object, x, method = NULL, full_layer = FALSE, sample_siz
       }
     } else {
       named_res <- list()
-      if ( "metadata" %in% names(object$specs) ) final_layer <- max(object$specs$metadata$Layer)
+      final_layer <- max(object$specs$metadata$Layer)
       for (k in 1:length(res)) {
-        if ( "metadata" %in% names(object$specs) ) {
           name1 <- subset(object$specs$metadata, object$specs$metadata[["Layer"]] == final_layer & object$specs$metadata[["Pos_in_Layer"]] == k)$Emulator
-        } else {
-          name1 <- paste('emulator', k, sep="")
-        }
         for (n in 1:dim(res[[k]])[1]) {
           name2 <- paste('output', n, sep="")
           named_res[[name1]][[name2]] <- res[[k]][n,,]
@@ -544,12 +346,13 @@ predict.lgp <- function(object, x, method = NULL, full_layer = FALSE, sample_siz
 #' @rdname predict
 #' @method predict gp
 #' @export
-predict.gp <- function(object, x, method = NULL, sample_size = 50, M = 50, cores = 1, chunks = NULL, ...) {
+predict.gp <- function(object, x, method = "mean_var", sample_size = 50, M = 50, cores = 1, chunks = NULL, ...) {
   if ( is.null(pkg.env$dgpsi) ) {
     init_py(verb = F)
     if (pkg.env$restart) return(invisible(NULL))
   }
   if ( !inherits(object,"gp") ) stop("'object' must be an instance of the 'gp' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   if ( !is.matrix(x)&!is.vector(x) ) stop("'x' must be a vector or a matrix.", call. = FALSE)
   if ( is.vector(x) ) {
     if ( ncol(object$data$X)!=1 ){
@@ -560,11 +363,7 @@ predict.gp <- function(object, x, method = NULL, sample_size = 50, M = 50, cores
   }
   if ( ncol(x) != ncol(object$data$X) ) stop("'x' must have the same number of dimensions as the training input.", call. = FALSE)
 
-  if ( is.null(method) ){
-    method = 'mean_var'
-  } else {
-    if ( method!='mean_var' & method!='sampling' ) stop("'method' can only be either 'mean_var' or 'sampling'.", call. = FALSE)
-  }
+  if ( method!='mean_var' & method!='sampling' ) stop("'method' can only be either 'mean_var' or 'sampling'.", call. = FALSE)
 
   sample_size <- as.integer(sample_size)
   M <- as.integer(M)
