@@ -1,10 +1,6 @@
 #' @title Locate the next design point for a (D)GP emulator or a bundle of (D)GP emulators using VIGF
 #'
-#' @description
-#'
-#' `r new_badge("updated")`
-#'
-#' This function searches from a candidate set to locate the next design point(s) to be added to a (D)GP emulator
+#' @description This function searches from a candidate set to locate the next design point(s) to be added to a (D)GP emulator
 #'     or a bundle of (D)GP emulators using the Variance of Improvement for Global Fit (VIGF). For VIGF on GP emulators, see the reference below.
 #'
 #' @param object can be one of the following:
@@ -15,20 +11,20 @@
 #'     from which the next design point(s) are determined. If `object` is an instance of the `bundle` class and `aggregate` is not supplied, `x_cand` can also be a list.
 #'     The list must have a length equal to the number of emulators in `object`, with each element being a matrix representing the candidate set for a corresponding
 #'     emulator in the bundle. Defaults to `NULL`.
-#' @param n_start `r new_badge("new")` an integer that gives the number of initial design points to be used to determine next design point(s). This argument
+#' @param n_start an integer that gives the number of initial design points to be used to determine next design point(s). This argument
 #'     is only used when `x_cand` is `NULL`. Defaults to `10`.
 #' @param batch_size an integer that gives the number of design points to be chosen.
 #'     Defaults to `1`.
-#' @param M `r new_badge("new")` the size of the conditioning set for the Vecchia approximation in the criterion calculation. This argument is only used if the emulator `object`
+#' @param M the size of the conditioning set for the Vecchia approximation in the criterion calculation. This argument is only used if the emulator `object`
 #'     was constructed under the Vecchia approximation. Defaults to `50`.
 #' @param workers the number of processes to be used for design point selection. If set to `NULL`,
 #'     the number of processes is set to `max physical cores available %/% 2`. Defaults to `1`. The argument does not currently support Windows machines when the `aggregate`
 #'     function is provided, due to the significant overhead caused by initializing the Python environment for each worker under spawning.
-#' @param limits `r new_badge("new")` a two-column matrix that gives the ranges of each input dimension, or a vector of length two if there is only one input dimension.
+#' @param limits a two-column matrix that gives the ranges of each input dimension, or a vector of length two if there is only one input dimension.
 #'     If a vector is provided, it will be converted to a two-column row matrix. The rows of the matrix correspond to input dimensions, and its
 #'     first and second columns correspond to the minimum and maximum values of the input dimensions. This
 #'     argument is only used when `x_cand = NULL`. Defaults to `NULL`.
-#' @param int `r new_badge("new")` a bool or a vector of bools that indicates if an input dimension is an integer type. If a single bool is given, it will be applied to
+#' @param int a bool or a vector of bools that indicates if an input dimension is an integer type. If a single bool is given, it will be applied to
 #'     all input dimensions. If a vector is provided, it should have a length equal to the input dimensions and will be applied to individual
 #'     input dimensions. This argument is only used when `x_cand = NULL`. Defaults to `FALSE`.
 #' @param aggregate an R function that aggregates scores of the VIGF across different output dimensions (if `object` is an instance
@@ -127,6 +123,7 @@ vigf.gp <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M = 50,
   }
   #check class
   if ( !inherits(object,"gp") ) stop("'object' must be an instance of the 'gp' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   training_input <- object$data$X
   training_output <- object$data$Y
   n_dim_X <- ncol(training_input)
@@ -248,6 +245,7 @@ vigf.dgp <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M = 50
   }
   #check class
   if ( !inherits(object,"dgp") ) stop("'object' must be an instance of the 'dgp' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulator or, if it was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   if ( object$constructor_obj$all_layer[[object$constructor_obj$n_layer]][[1]]$type != 'likelihood' & !is.null(object$constructor_obj$indices) ){
     stop("The function is not applicable to DGP emulators whose training data contain replicates but are without likelihood layers.", call. = FALSE)
   }
@@ -596,6 +594,7 @@ vigf.bundle <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M =
   }
   #check class
   if ( !inherits(object,"bundle") ) stop("'object' must be an instance of the 'bundle' class.", call. = FALSE)
+  if ( reticulate::py_is_null_xptr(object$emulator1$constructor_obj) ) stop("The Python session originally associated with 'object' is no longer active. Please rebuild the emulators in the bundle or, if the bundle was saved using dgpsi::write(), load it into the R session with dgpsi::read().", call. = FALSE)
   #check no of emulators
   n_emulators <- length(object)
   if ( "data" %in% names(object) ) n_emulators <- n_emulators - 1
@@ -695,7 +694,7 @@ vigf.bundle <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M =
             initials = x_cand[[i]],
             lb = reticulate::np_array(lower),
             up = reticulate::np_array(upper),
-            args = reticulate::tuple('VIGF', obj_i$constructor_obj, 1., M, TRUE),
+            args = if ( inherits(obj_i,"gp") ) {reticulate::tuple('VIGF', 1., M, TRUE)} else {reticulate::tuple('VIGF', obj_i$constructor_obj, 1., M, TRUE)},
             method = "L-BFGS-B",
             core_num = workers,
             out_dim = -1L,
@@ -731,7 +730,11 @@ vigf.bundle <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M =
             x[int] <- round(x[int])
             for (i in 1:n_emulators){
               obj_i <- object[[paste('emulator',i,sep='')]]
-              scores_i <- obj_i$emulator_obj$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', obj = obj_i$constructor_obj, m = m, score_only = TRUE)
+              if ( inherits(obj_i,"gp") ){
+                scores_i <- obj_i$emulator_obj$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', m = m, score_only = TRUE)
+              } else {
+                scores_i <- obj_i$emulator_obj$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', obj = obj_i$constructor_obj, m = m, score_only = TRUE)
+              }
               scores[[i]] <- if(ncol(scores_i) == 1) as.vector(scores_i) else rowMeans(scores_i)
             }
             scores <- do.call(cbind, scores)
@@ -757,7 +760,11 @@ vigf.bundle <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M =
             x[int] <- round(x[int])
             for (i in 1:n_emulators){
               obj_i <- object[[paste('emulator',i,sep='')]]
-              scores_i <- obj_i$emulator_obj$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', obj = obj_i$constructor_obj, m = m, score_only = TRUE)
+              if ( inherits(obj_i,"gp") ){
+                scores_i <- obj_i$emulator_obj$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', m = m, score_only = TRUE)
+              } else {
+                scores_i <- obj_i$emulator_obj$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', obj = obj_i$constructor_obj, m = m, score_only = TRUE)
+              }
               scores[[i]] <- if(ncol(scores_i) == 1) as.vector(scores_i) else rowMeans(scores_i)
             }
             scores <- do.call(cbind, scores)
@@ -867,7 +874,7 @@ vigf.bundle <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M =
               initials = x_cand[[j]],
               lb = reticulate::np_array(lower),
               up = reticulate::np_array(upper),
-              args = reticulate::tuple('VIGF', constructor_obj_list[[j]], 1., M, TRUE),
+              args = if ( inherits(object[[paste('emulator',j,sep='')]],"gp") ) {reticulate::tuple('VIGF', 1., M, TRUE)} else {reticulate::tuple('VIGF', constructor_obj_list[[j]], 1., M, TRUE)},
               method = "L-BFGS-B",
               core_num = workers,
               out_dim = -1L,
@@ -886,7 +893,11 @@ vigf.bundle <- function(object, x_cand = NULL, n_start = 10, batch_size = 1, M =
             scores <- vector('list', n_emulators)
             x[int] <- round(x[int])
             for (k in 1:n_emulators){
-              scores_k <- emulator_obj_list[[k]]$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', obj = constructor_obj_list[[k]], m = m, score_only = TRUE)
+              if ( inherits(object[[paste('emulator',k,sep='')]],"gp") ){
+                scores_k <- emulator_obj_list[[k]]$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', m = m, score_only = TRUE)
+              } else{
+                scores_k <- emulator_obj_list[[k]]$metric(x_cand = matrix(x, nrow = 1), method = 'VIGF', obj = constructor_obj_list[[k]], m = m, score_only = TRUE)
+              }
               scores[[k]] <- if(ncol(scores_k) == 1) as.vector(scores_k) else rowMeans(scores_k)
             }
             scores <- do.call(cbind, scores)
